unit aqrreg;
{
  Quick Report Editor V2.1

  Copyright (c) 1997,2000 Demidov Andre (andre@lion.susu.ac.ru)

}

interface
uses
  Classes;


procedure Register;

implementation

uses
  qrepedit, SysUtils, exptintf, dsgnintf;
{$I QREPDEFS.INC}

{$IFDEF WIN32}
{$R *.D32}
{$ELSE}
{$R *.D16}
{$ENDIF}

{$IFDEF QR20}
type
  TAQRCellEditor = class(TComponentEditor)
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): string; override;
    function GetVerbCount: Integer; override;
  end;

procedure TAQRCellEditor.ExecuteVerb(Index: Integer);
begin
  case Index of
  0: begin
      TAQRCell(Component).UpdateSizes;
     end;
  1: begin
      TAQRCell(Component).Tie;
     end;
  2: begin
      TAQRCell(Component).UnTie;
     end;
  end;
end;

function TAQRCellEditor.GetVerb(Index: Integer): string;
begin
  case Index of
  0: Result := 'Update';
  1: Result := 'Tie';
  2: Result := 'Untie';
  end;
end;

function TAQRCellEditor.GetVerbCount: Integer;
begin
  Result := 3;
end;
{$ENDIF}

type
  TFieldListProperty = class(TStringProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValueList(List: TStrings); virtual; abstract;
    procedure GetValues(Proc: TGetStrProc); override;
  end;


function TFieldListProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList,paMultiSelect];
end;

procedure TFieldListProperty.GetValues(Proc: TGetStrProc);
var
  I: Integer;
  Values: TStringList;
begin
  Values := TStringList.Create;
  try
    GetValueList(Values);
    for I := 0 to Values.Count - 1 do Proc(Values[I]);
  finally
    Values.Free;
  end;
end;

type
  TFieldFormatProperty = class(TFieldListProperty)
  public
    procedure GetValueList(List: TStrings); override;
  end;

procedure TFieldFormatProperty.GetValueList(List: TStrings);
var
   n,i,l:integer;
   AQREd:TAQREditorFields;
   fmt:string[4];
   nam:string[32];
   cmp:TComponent;
begin
  AQREd:=nil;
  cmp:=GetComponent(0) as TComponent;
  with cmp.Owner do
    for i:=0 to ComponentCount-1 do
      if Components[i] is TAQREditorFields then
      begin
        AQREd:=TAQREditorFields(Components[i]);
        break;
      end;
  nam:='';
  if cmp is TAQRLabel then
    nam:=TAQRLabel(cmp).FieldName
  else if cmp is TAQRMemo then
    nam:=TAQRMemo(cmp).FieldName;
  List.Clear;
  fmt:='';
  if AQREd<>nil then
  begin
    with AQREd.FieldNames do
      for i:=0 to count-1 do
      begin
        l:=Pos(':',Strings[i]);
        if l=0 then
          l:=Length(Strings[i])
        else
          Dec(l);
        if Copy(Strings[i],1,L)=nam then
        begin
          fmt:=Copy(Strings[i],L+2,4);
          break;
        end;
      end;
    with AQREd.FieldFormats do
      for i:=0 to count-1 do
      begin
        l:=Pos(':',Strings[i]);
        if l<>0 then
          if Copy(Strings[i],1,L-1)=fmt then
          begin
            List.Add(Copy(Strings[i],L+1,100));
          end;
      end;
  end;
end;

type
  TFieldNameProperty = class(TFieldListProperty)
  public
    function GetAttributes: TPropertyAttributes; override;
    procedure GetValueList(List: TStrings); override;
  end;

function TFieldNameProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paValueList,paSortList,paMultiSelect];
end;

procedure TFieldNameProperty.GetValueList(List: TStrings);
var
   n,i,l:integer;
   AQREd:TAQREditorFields;
begin
  AQREd:=nil;
  with (GetComponent(0) as TComponent).Owner do
    for i:=0 to ComponentCount-1 do
      if Components[i] is TAQREditorFields then
      begin
        AQREd:=TAQREditorFields(Components[i]);
        break;
      end;
  List.Clear;
  if AQREd<>nil then
  begin
    with AQREd.FieldNames do
      for i:=0 to count-1 do
      begin
        l:=Pos(':',Strings[i]);
        if l=0 then
          l:=Length(Strings[i])
        else
          Dec(l);
        List.Add(Copy(Strings[i],1,L));
      end;
  end;
end;

procedure Register;
begin
  RegisterComponents(
{$IFDEF QR20}
  'QReport',
{$ELSE}
  'QuickReport',
{$ENDIF}
    [TAQREditor,TAQRBand,TAQRLabel,TAQRShape,TAQRMemo,TAQREDitorFields
{$IFDEF QR20}
    ,TAQRCell,TAQRImage
{$ENDIF}
    ]);
  RegisterPropertyEditor(TypeInfo(string32), TAQRLabel, 'FieldName',TFieldNameProperty);
  RegisterPropertyEditor(TypeInfo(string32), TAQRLabel, 'FieldFormat',TFieldFormatProperty);
  RegisterPropertyEditor(TypeInfo(string32), TAQRMemo, 'FieldName',TFieldNameProperty);
  RegisterPropertyEditor(TypeInfo(string32), TAQRMemo, 'FieldFormat',TFieldFormatProperty);
{$IFDEF QR20}
  RegisterComponentEditor(TAQRCell, TAQRCellEditor);
{$ENDIF}
end;

end.
